@@ -49,7 +49,7 @@ for(i in 1:1)# change to 7 in order to perform all tests
   
   #print(x)
   
-  ElectreAssignments<-LPDMRSort(performanceTable, x$profilesPerformances, 
+  ElectreAssignments<-LPDMRSort(performanceTable, x$profilesPerformances, categoriesRanks,
                                 x$weights, criteriaMinMax, x$lambda, criteriaVetos=x$vetoPerformances, criteriaDictators=x$dictatorPerformances, majorityRule = majorityRules[i],
                                 alternativesIDs = filteredAlternativesIDs)
   
